package it.openutils.spring.editors;

import java.beans.PropertyEditorSupport;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

import org.apache.commons.beanutils.BeanUtils;
import org.apache.commons.beanutils.MethodUtils;
import org.apache.commons.lang.StringUtils;
import org.apache.log4j.Logger;


/**
 * @author fgiust
 * @version $Revision$ ($Author$)
 */
public class GenericSetPropertyEditor extends PropertyEditorSupport
{

    /**
     * Logger.
     */
    private static Logger log = Logger.getLogger(GenericSetPropertyEditor.class);

    /**
     * Generic data access object.
     */
    private Object dao;

    /**
     * property name in object
     */
    private String propertyName;

    /**
     * property type for object id
     */
    private Class propertyType;

    /**
     * ProfiloPropertyEditor.
     * @param dao generic DAO
     * @param propertyName property name
     * @param propertyType property type
     */
    public GenericSetPropertyEditor(Object dao, String propertyName, Class propertyType)
    {
        this.dao = dao;
        this.propertyName = propertyName;
        this.propertyType = propertyType;
    }

    /**
     * @see java.beans.PropertyEditorSupport#getAsText()
     */
    public String getAsText()
    {
        Set value = (Set) getValue();

        if (log.isDebugEnabled())
        {
            log.debug("getAsText(" + value + ")");
        }

        if (value != null && value.size() > 0)
        {
            try
            {
                StringBuffer buffer = new StringBuffer();
                Iterator iterator = value.iterator();
                while (iterator.hasNext())
                {
                    Object item = iterator.next();

                    buffer.append(BeanUtils.getProperty(item, propertyName));
                    if (iterator.hasNext())
                    {
                        buffer.append(",");
                    }

                }

                return buffer.toString();
            }
            catch (Exception e)
            {
                log.error(e.getMessage(), e);
            }
        }

        return StringUtils.EMPTY;
    }

    /**
     * @see java.beans.PropertyEditorSupport#setAsText(java.lang.String)
     */
    public void setAsText(String text) throws IllegalArgumentException
    {

        if (log.isDebugEnabled())
        {
            log.debug("setAsText(" + text + ")");
        }

        if (StringUtils.isNotEmpty(text))
        {
            Set set = new HashSet();
            String[] values = StringUtils.split(text, ',');
            for (int j = 0; j < values.length; j++)
            {
                String id = values[j];
                if (StringUtils.isNotEmpty(id))
                {
                    try
                    {
                        Object[] parameters = new Object[1];

                        if (Integer.class.equals(this.propertyType))
                        {
                            parameters[0] = new Integer(id);
                        }
                        if (String.class.equals(this.propertyType))
                        {
                            parameters[0] = id;
                        }
                        if (Long.class.equals(this.propertyType))
                        {
                            parameters[0] = new Long(id);
                        }

                        Object value = MethodUtils
                            .invokeMethod(dao, "load", parameters, new Class[]{this.propertyType});
                        set.add(value);
                    }
                    catch (Throwable e)
                    {
                        log.error("Not adding object with id [" + id + "] due to a " + e.getClass().getName(), e);
                    }
                }
            }
            setValue(set);
        }
    }
}
