package it.openutils.spring.editors;

import java.beans.PropertyEditorSupport;

import org.apache.commons.beanutils.BeanUtils;
import org.apache.commons.beanutils.MethodUtils;
import org.apache.commons.lang.ClassUtils;
import org.apache.commons.lang.StringUtils;
import org.apache.log4j.Logger;


/**
 * @author fgiust
 * @version $Revision$ ($Author$)
 */
public class GenericSinglePropertyEditor extends PropertyEditorSupport
{

    /**
     * Logger.
     */
    private static Logger log = Logger.getLogger(GenericSinglePropertyEditor.class);

    /**
     * Generic data access object.
     */
    private Object dao;

    /**
     * property name in object
     */
    private String propertyName;

    /**
     * property type for object id
     */
    private Class propertyType;

    /**
     * ProfiloPropertyEditor.
     * @param dao generic DAO
     * @param propertyName property name
     * @param propertyType property type
     */
    public GenericSinglePropertyEditor(Object dao, String propertyName, Class propertyType)
    {
        this.dao = dao;
        this.propertyName = propertyName;
        this.propertyType = propertyType;
    }

    /**
     * @see java.beans.PropertyEditorSupport#getAsText()
     */
    public String getAsText()
    {
        Object value = getValue();
        if (log.isDebugEnabled())
        {
            log.debug("getAsText(" + value + ")");
        }

        String returnString = null;

        if (value != null)
        {
            try
            {
                returnString = BeanUtils.getProperty(value, propertyName);
            }
            catch (Exception e)
            {
                log.error(e.getMessage(), e);
            }
        }

        return StringUtils.defaultString(returnString);
    }

    /**
     * @see java.beans.PropertyEditorSupport#setAsText(java.lang.String)
     */
    public void setAsText(String text) throws IllegalArgumentException
    {
        log.debug("setAsText(" + text + ")");

        if (StringUtils.isNotBlank(text))
        {

            try
            {
                Object[] parameters = new Object[1];

                if (Integer.class.equals(this.propertyType))
                {
                    parameters[0] = new Integer(text);
                }
                if (String.class.equals(this.propertyType))
                {
                    parameters[0] = text;
                }
                if (Long.class.equals(this.propertyType))
                {
                    parameters[0] = new Long(text);
                }

                Object value = MethodUtils.invokeMethod(dao, "load", parameters, new Class[]{this.propertyType});
                setValue(value);
            }
            catch (Exception e)
            {
                log.error(e.getMessage(), e);
            }

        }
    }

    /**
     * Set (or change) the object that is to be edited.
     * @param value The new target object to be edited. Note that this object should not be modified by the
     * PropertyEditor, rather the PropertyEditor should create a new object to hold any modified value.
     */
    public void setValue(Object value)
    {
        log.debug("*** setValue(" + ClassUtils.getShortClassName(value, "<null>") + ")");
        super.setValue(value);
    }
}
